package ij.plugin;
import java.awt.*;
import java.io.*;
import ij.*;
import ij.io.*;

/**	Opens and displays FITS images. The FITS format is
	described at "ftp://nssdc.gsfc.nasa.gov/pub/fits".
*/
public class FITS extends ImagePlus implements PlugIn {

	public void run(String arg) {
		OpenDialog od = new OpenDialog("Open FITS...", arg);
		String directory = od.getDirectory();
		String fileName = od.getFileName();
		if (fileName==null)
			return;
		IJ.showStatus("Opening: " + directory + fileName);
		FitsDecoder decoder = new FitsDecoder(directory, fileName);
		FileInfo fi = null;
		try {fi = decoder.getInfo();}
		catch (IOException e) {}
		if (fi!=null && fi.width>0 && fi.height>0 && fi.offset>0) {
			FileOpener fo = new FileOpener(fi);
			ImagePlus imp = fo.open(false);
			setProcessor(fileName, imp.getProcessor());
			setProperty("Info", decoder.getHeaderInfo());
			if (arg.equals("")) show();
		} else
			IJ.error("This does not appear to be a FITS file.");
		IJ.showStatus("");
	}
}

class FitsDecoder {
	private String directory, fileName;
	private DataInputStream f;
	private StringBuffer info = new StringBuffer(512);

	public FitsDecoder(String directory, String fileName) {
		this.directory = directory;
		this.fileName = fileName;
	}

	String getString(int length) throws IOException {
		byte[] b = new byte[length];
		f.read(b);
		return new String(b);
	}

	int getInteger(String s) {
		s = s.substring(10, 30);
		s = s.trim();
		return Integer.parseInt(s);
	}

	FileInfo getInfo() throws IOException {
		FileInfo fi = new FileInfo();
		fi.fileFormat = fi.RAW;
		fi.fileName = fileName;
		fi.directory = directory;
		fi.width = 0;
		fi.height = 0;
		fi.offset = 0;
		
		f = new DataInputStream(new FileInputStream(directory + fileName));
		String s = getString(80);
		info.append(s+"\n");
		if (!s.startsWith("SIMPLE"))
			{f.close(); return null;}
		int count = 1;
		do {
			count++;
			s = getString(80);
			info.append(s+"\n");
			if (s.startsWith("BITPIX")) {
				int bitsPerPixel = getInteger(s);
				if (bitsPerPixel==8)
					fi.fileType = FileInfo.GRAY8;
				else if (bitsPerPixel==16)
					fi.fileType = FileInfo.GRAY16_SIGNED;
				else if (bitsPerPixel==32)
					fi.fileType = FileInfo.GRAY32_INT;
				else if (bitsPerPixel==-32)
					fi.fileType = FileInfo.GRAY32_FLOAT;
				else {
					IJ.error("BITPIX must be 8, 16, 32 or -32 (float).");
					f.close();
					return null;
				}
			}else if (s.startsWith("NAXIS1"))
				fi.width = getInteger(s);
			else if (s.startsWith("NAXIS2"))
				fi.height = getInteger(s);
			if (count>360)
				{f.close(); return null;}
		} while (!s.startsWith("END"));
		fi.offset = 2880+2880*((count*80)/2880);
		f.close();
		return fi;
	}

	String getHeaderInfo() {
		return new String(info);
	}

}
