package ij.plugin.frame;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import ij.*;
import ij.plugin.*;
import ij.process.*;
import ij.gui.*;

/** Image Processing Demo. Demonstrates the use of ImageJ's ImageProcessor class. */
public class IP_Demo extends PlugInFrame implements PlugIn, ActionListener {

	Panel panel;
	int previousId;

	public IP_Demo() {
		super("IP Demo");
		setLayout(new FlowLayout());
		panel = new Panel();
		panel.setLayout(new GridLayout(4, 4, 5, 5));
		addButton("Reset");
		addButton("Flip");
		addButton("Invert");
		addButton("Rotate");
		addButton("Lighten");
		addButton("Darken");
		addButton("Zoom In");
		addButton("Zoom Out");
		addButton("Smooth");
		addButton("Sharpen");
		addButton("Find Edges");
		addButton("Threshsold");
		addButton("Add Noise");
		addButton("Reduce Noise");
		addButton("Macro1");
		addButton("Macro2");
		add(panel);
		
		pack();
		GUI.center(this);
		show();
	}
	
	
	void addButton(String label) {
		Button b = new Button(label);
		b.addActionListener(this);
		panel.add(b);
	}

	public void actionPerformed(ActionEvent e) {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null) {
			IJ.showStatus("No image");
			previousId = 0;
			return;
		}
		if (!imp.lock())
			{previousId = 0; return;}
		ImageProcessor ip = imp.getProcessor();
		if (imp.getID()!=previousId)
	 		ip.snapshot();
	 	previousId = imp.getID();
	 	
		String label = e.getActionCommand();
		if (label==null)
			return;
		new IPExecuter(label, imp, ip);
	}
	
	//public void dispose() {
	//	IJ.write("IP demo: dispose");
	//	super.dispose();
	//}

}

/** Runs IP Demo commands in a separate thread. */
class IPExecuter extends Thread {

	private String command;
	private ImagePlus imp;
	private ImageProcessor ip;
	
	IPExecuter(String command, ImagePlus imp, ImageProcessor ip) {
		super(command);
		this.command = command;
		this.imp = imp;
		this.ip = ip;
		setPriority(Math.max(getPriority()-2, MIN_PRIORITY));
		start();
	}

	public void run() {
		try {runCommand(command, imp, ip);}
		catch(OutOfMemoryError e) {
			IJ.outOfMemory(command);
			if (imp!=null) imp.unlock();
		} catch(Exception e) {
			CharArrayWriter caw = new CharArrayWriter();
			PrintWriter pw = new PrintWriter(caw);
			e.printStackTrace(pw);
			IJ.write(caw.toString());
			IJ.showStatus("");
			if (imp!=null) imp.unlock();
		}
	}

    public void runCommand(String name, ImagePlus imp, ImageProcessor ip) {
		IJ.showStatus(command + "...");
		if (command.equals("Reset"))
			ip.reset();
		else if (command.equals("Flip"))
			ip.flipVertical();
		else if (command.equals("Invert"))
			ip.invert();
		else if (command.equals("Lighten")) {
			if (imp.isInvertedLut())
				ip.multiply(0.9);
			else
				ip.multiply(1.1);
		}
		else if (command.equals("Darken")) {
			if (imp.isInvertedLut())
				ip.multiply(1.1);
			else
				ip.multiply(0.9);
		}
		else if (command.equals("Rotate"))
			ip.rotate(30);
		else if (command.equals("Zoom In"))
			ip.scale(1.2, 1.2);
		else if (command.equals("Zoom Out"))
			ip.scale(.8, .8);
		else if (command.equals("Threshsold"))
			ip.autoThreshold();
		else if (command.equals("Smooth"))
			ip.smooth();
		else if (command.equals("Sharpen"))
			ip.sharpen();
		else if (command.equals("Find Edges"))
			ip.findEdges();
		else if (command.equals("Add Noise"))
			ip.noise(20);
		else if (command.equals("Reduce Noise"))
			ip.medianFilter();
		else if (command.equals("Macro1"))
			macro1(imp, ip);
		else if (command.equals("Macro2"))
			macro2(imp, ip);
		Roi roi = imp.getRoi();
		if (roi!=null) {
			int[] mask = roi.getMask();
			if (mask!=null)
				ip.reset(mask);
		}
		imp.updateAndDraw();
		imp.unlock();
		IJ.showStatus("");
    }

	void macro1(ImagePlus imp, ImageProcessor ip) {
		for (int i=10; i<=360; i+=10) {
			ip.reset();
			ip.rotate(i);
			imp.updateAndDraw();
		}
	}
	
	void macro2(ImagePlus imp, ImageProcessor ip) {
		double scale = 1, m = 1.2;
		for (int i=0; i<20; i++) {
			ip.reset();
			scale *= m;
			ip.scale(scale, scale);
			imp.updateAndDraw();
		}
		for (int i=0; i <20; i++) {
			ip.reset();
			scale /= m;
			ip.scale(scale, scale);
			imp.updateAndDraw();
		}
	}

}


